#!/usr/bin/env python
""" Test reading an hdf5 file
"""

import os
import sys
from   pylab import *
from   scipy.stats import norm as gaus
from   scipy import optimize
import h5py
import time
import math
from optparse import OptionParser
from numpy import sum


def fLandau(p, x):
    """ Approximate the landau distribution with the Moyal
        distribution
        
        Parameters:
            p[0] = normalization
            p[1] = mode
            p[2] = width
    """
    L = (x-p[1])/p[2]
    r = p[0]*exp(-0.5*(L+exp(-L)))/2.506628274631001 #math.sqrt(2.0*math.pi)
    return r

def fitLandau(p, x, y):
    """ This is the 'error' function we use for
        the fit.
    """
    return fLandau(p, x) - y


def fgauss(p, x):
    """ This is a gaussion.
        Parameters:
           p[0] = maximum
           p[1] = mean
           p[2] = sigma
    """
    L  = (x-p[1])/p[2]
    L1 = p[0]*exp(-0.5*L*L)/(2.506628274631001*p[2])
    return L1

def fitGauss(p, x, y):
    """ The error function
    """
    return fgauss(p, x) - y


class GTimer(object):
    """ Implements the GLib GTimer
    """
    def __init__(self):
        self._start = 0
        self._end = 0
        self._running = False
        
    def start(self):
        self._start = time.time()
        
    def __call__(self):
        if self._running:
            return time.time() - self._start
        
        else:
            return 0.0
        
    def start(self):
        self._start = time.time()
        self._running = True
        
    def stop(self):
        self._running = False
        
    def reset(self):
        if self._running:
            self._start = time.time()
            

def process_event(ievt, S, nchan, nchip, pedestal, noise):
    """ Process a single event
        Returns a tuple with the signal, signal/noise and common mode per chip
    """
    
    # signal without pedestals
    ss = single(S[ievt,:]) - pedestal
    
    # compute ans subtract common mode
    cm = zeros(nchip)
    
    for i in range(0,nchip):
        cm[i] = mean(ss[128*i:128*(i+1)])
        ss[128*i:128*(i+1)] -= cm[i]
        
    sn = ss/noise
    
    return ss, sn, cm


def main(fname, fit_landau=True):
    """ Analyze file fname
    """
    
    # Open the file
    if not os.path.exists(fname):
        print "Input file", fname,"does not exist"
        return
    
    
    F = h5py.File(fname, 'r')
    
    
    # Get pedestals and noise
    pedestal = F['/header/pedestal']
    noise    = F['/header/noise']
    
    # Get the signal array
    S = F['/events/signal']
    
    # Test that pedestals are OK
    P = mean(S[0:], axis=0)
    pdiff = P - pedestal
    pm = mean(pdiff)
    print "Average pedestal difference:", pm
    
    # Convert pedestal and noise to "vectors"
    pedestal = pedestal[0,:]
    noise    = noise[0,:]
    sz       = pedestal.shape
    nchan    = sz[0]
    nchip    = nchan/128

    #
    # Now process the whole file
    #
    # Get the events with good timing
    print "Get Events with good timing"
    T     = F['/events/time']
    GT    = nonzero(T[:]>5.0)
    nevts = GT[0].shape[0]
    
    # Reduce the data sample to those events
    print "Loop on Events. Nevt=", nevts
    TT = GTimer() # This is to show the number of events processed
    TG = GTimer() # This is to profile the process
    
    Ttot = 0.0
    #
    # This is to keep the cluster energy
    #
    G = zeros(0) 
    Gnc = zeros(0)
    
    #
    # This is to keep the common mode of each event
    #
    C = [zeros(i) for i in range(0, nchip)] 
     
    #
    # Start the timers and the main loop
    #                                        
    TT.start()
    TG.start()
    for ic, ievt in enumerate(GT[0]):
        if TT() > 1.0:
            print "\r%10d, %10d" % (ic, ievt),
            sys.stdout.flush()
            TT.reset()
            
        # Process the event
        ss, sn, cm = process_event(ievt, S, nchan, nchip, pedestal, noise)
        
        # Append the commpn mode.
        for i in range(0, nchip):
            C[i] = append(C[i], cm[i])
            
        #
        # find the clusters
        #
        used = [ False for x in range(0, nchan) ]
        channels = nonzero(sn>5.0)
        for ch in channels[0]:
            clst = [ch]
            used[ch] = True
            for i in range(ch-1, 0, -1):
                if sn[i]>3.0 and not used[i]:
                    clst.append(i)
                    used[i] = True
                else:
                    break
                
            for i in range(ch+1, nchan):
                if sn[i]>3.0 and not used[i]:
                    clst.append(i)
                    used[i] = True
                else:
                    break
                
            Eclst = 0.0
            for i in clst:
                if ss[i]>0.0:
                    Eclst += ss[i]
        
            if abs(Eclst)/noise[ch] > 5.0:
                G = append(G, Eclst)
        
        sz = size(channels)
        if sz == 1: # we do not know how to handle clusters
            for ic in range(0, sz):
                Gnc = append(Gnc, ss[channels[ic]])
                
    TT.stop()    
    print "\nTotal time elapsed:", TG()
    
    
    # PLot some quantities
    figure(1)
    
    # The common mode
    X = r_[-15:15:0.5]
    for i in range(0, nchip):
        subplot(3,nchip,i+1)
        xlabel("common mode chip %d (ADC)" % i)    
        n, bins, patches = hist(C[i],X, normed=1)
        mu, sigma = gaus.fit(C[i])
        y = normpdf(bins, mu, sigma)
        l = plot(bins, y, 'r-', linewidth=2)
        p = Rectangle((0, 0), 1, 1, fc="r")
        legend([p], [r'$\mu$=%.3f $\sigma$=%.1f' %( mu, sigma)], loc=1)
    
    
    # Data histogram (the landau)
    subplot(3,1,2)
    X = r_[1:500:5]
    xlabel("Signal (ADC)")
    n, bins, patches = hist(G,X)
    
    indx = argmax(n, 0)
    if fit_landau:
        errFunc = fitLandau
        dataFunc = fLandau
        param = [ 0.25*sum(n), bins[indx], 10. ]
    else:
        errFunc = fitGauss
        dataFunc = fgauss
        param = [ sum(n), bins[indx], 10. ]

    # Fit to the landau distribution
    step = 0.5*(bins[1]-bins[0])
    X = bins[:-1] + step
    fit_rc = optimize.leastsq(errFunc, param[:], args=(X, n))
    fit_par = fit_rc[0]

    # Draw the fitted curve on top of the histogram
    y = dataFunc(fit_par, X)
    l = plot(X, y, 'r-', linewidth=2, label="hola")
    legend([p],[r'mode  %.1f sigma %.1f' % (fit_par[1], fit_par[2])])
    
    subplot(3,1,3)
    xlabel("Signal - No cluster (ADC)")
    n, bins, patches = hist(Gnc,X)
    
    
    tight_layout()
    show()

if __name__ == "__main__":
    parser = OptionParser()
    parser.add_option("--gauss",
                      dest="gaus", action="store_true",
                      help="It will try a gauss fit to the data",
                      default=False
                      )    
    parser.add_option("--landau",
                      dest="landau", action="store_true",
                      help="It will try a Landau-like fit to the data",
                      default=True
                      )    
    (options, args) = parser.parse_args()
    if options.gaus:
        options.landau = False
    
    try:
        main(args[0], options.landau)
    except KeyError:
        print "I need an input file"