/*
 * ConfigFactory.cc
 *
 *  Created on: May 17, 2010
 *      Author: lacasta
 */
#include <iostream>
#include <string>
#include <cstdlib>
#include <cerrno>
#include <limits>
#include "ConfigFactory.h"
#include <PluginConfig.h>
#include <ScanConfig.h>
#include <BeetleConfig.h>
#include <LaserConfig.h>
#include <TriggerConfig.h>
#include <DAQConfig.h>
#include <ComparatorConfig.h>
#include <Analysis.h>
#include <Alibava.h>
#include <DAQmanager.h>
#include "utils.h"
#include "string_utils.h"

typedef std::map<std::string, ConfigFile *> ConfigMap;
static ConfigFactory *__the_factory__ = 0;

ConfigFactory::ConfigFactory()
{
    config_map["calibration"] = new CalibrationConfig();
    config_map["lasersync"] = new LaserSyncConfig(100, 0, 1000);
    config_map["beetle"] = new BeetleConfig();
    config_map["laser"] = new LaserConfig();
    config_map["trigger"] = new TriggerConfig();
    config_map["daq"] = new DAQConfig();
    config_map["plugin"] = new PluginConfig();
    config_map["analysis"] = new Analysis();
}

ConfigFactory::~ConfigFactory()
{
    ConfigMap::iterator ip;
    for (ip = config_map.begin(); ip!=config_map.end(); ip++)
    {
        delete ip->second;
    }
    config_map.clear();
}

ConfigFactory *ConfigFactory::instance()
{
    if (!__the_factory__)
        __the_factory__ = new ConfigFactory();

    return __the_factory__;
}

void ConfigFactory::set_file(KeyFile *key_file)
{
    ConfigMap::iterator ip;
    for (ip = config_map.begin(); ip!=config_map.end(); ip++)
    {
        ip->second->set_file(key_file);
    }
}

void ConfigFactory::save_data()
{
    ConfigMap::iterator ip;
    for (ip = config_map.begin(); ip!=config_map.end(); ip++)
    {
        ip->second->save_data();
    }
}

ConfigFile *ConfigFactory::get(const std::string &name)
{
    ConfigMap::iterator ip = config_map.find(name);
    if (ip!=config_map.end())
        return ip->second;
    else
        return 0;
}
int ConfigFactory::config_beetle(Alibava &B)
{
	int rc = 0;
    BeetleConfig *beetle = dynamic_cast<BeetleConfig *>(ConfigFactory::instance()->get("beetle"));
    unsigned short *val = new unsigned short[beetle->size() ];
    BeetleConfig::VarList::const_iterator ip;
    int nval=0;

    for (ip=beetle->begin(); ip!=beetle->end(); ++ip)
    {
        val[nval++] = ip->second;
    }

    // Set common Registers
    write_msg(_("*** Sending Beetle configuration\n"));
    if (beetle->config_beetle(nval, val, &B))
//    if (B.beetle_config(nval, beetle->mask(), val)<0)
    {
        delete [] val;
        return -1;
    }
    delete [] val;
    return 0;
}



int ConfigFactory::config_beetle_comparator(Alibava &B)
{
    BeetleConfig *beetle = dynamic_cast<BeetleConfig *>(ConfigFactory::instance()->get("beetle"));
    return beetle->config_beetle_comparator(&B);
}

int ConfigFactory::config_trigger(Alibava &B)
{
    write_msg(_("*** Sending Trigger configuration\n"));
    TriggerConfig *tc = dynamic_cast<TriggerConfig *>(ConfigFactory::instance()->get("trigger"));

    // Thresholds go in DAC units, so we have to translate
    return B.trigger(tc->pulse() ? Alibava::TriggerPulse : Alibava::TriggerIn,
                          tc->trigger_and() ? Alibava::TriggerAND : Alibava::TriggerOR,
                          ((tc->dacA()+2048) & 0xfff),
                          ((tc->dacB()+2048) & 0xfff)| 0x1000,
                          ((tc->dacC()+2048) & 0xfff)| 0x2000,
                          ((tc->dacD()+2048) & 0xfff)| 0x3000
                          );
}

int ConfigFactory::config_board(Alibava &B)
{
    int rc;


    DAQConfig *daq = dynamic_cast<DAQConfig *>(get("daq"));
    B.delay(daq->delay());
    B.config_firmware( daq->firmware_config());

    if ( (rc=config_beetle(B)) )
        return rc;

    if ( (rc=config_beetle_comparator(B)) )
        return rc;

    rc = config_trigger(B);
    return rc;
}

/*
 * Sets Beetle parameters via SOAP
 */
static int get_int_value(const std::string &s, int def=0)
{
    char *endptr;
    const char *str = s.c_str();
    long val = strtol(str, &endptr, 10);

    if (endptr == str)
    {
        return def;
    }

    return val;
}
static int get_int_array_value(const std::string &input, std::vector<int> &v)
{
    std::string val = trim_str(input);
    v.clear();
    std::string::size_type S=val.size();
    std::string::size_type i=0;
    while (i<S)
    {
        std::string::size_type j = val.find_first_of(",;|", i);
        int rc = get_int_value(val.substr(i,j-i), std::numeric_limits<int>::min());
        if (rc==std::numeric_limits<int>::min())
        {
            v.clear();
            return -1;
        }
        v.push_back(rc);

        if (j==val.npos)
            break;
        i = j+1;
    };
    return 0;
}


std::string triggerParam("trgIn.trgPulse.thrsIn1.thrsIn2.thrsPulse-.thrsPulse+.trgAND.trgOR.");
std::string compParam("enableComp.trackMode.pulseMode.compPolarity."
                      "mainTh.mainTh1.mainTh2.deltaTh.deltaTh1.deltaTh2."
                      "trimCh1.trimCh2.maskCh1.maskCh2."
    );

int set_parameter_value(const std::string &name, const std::string &value)
{
    int iok;
    ConfigFactory *config = ConfigFactory::instance();
    BeetleConfig *beetle = dynamic_cast<BeetleConfig *>(config->get("beetle"));
    DAQmanager *mngr = DAQmanager::get_the_manager();
    if (!mngr)
        return -1;


    /**
     * File format
     */
    if ( name == "fileFormat" )
    {
        std::string the_val = lower_string(value);
        if ( the_val.find("hdf") != the_val.npos )
            mngr->file_format(DataFile::HDF5);
        else
            mngr->file_format(DataFile::AsciiFile);

        return 0;
    }

    //std::cout << "Parameter " << name << " value " << value << std::endl;
    int val = get_int_value(value, std::numeric_limits<int>::min());
    if (val==std::numeric_limits<int>::min()  )
    {
        if (  name.find("trimCh")==name.npos && name.find("maskCh")==name.npos)
        {
            err_msg("Wrong value for parameter %s [%s]", name.c_str(), value.c_str());
            return -2;
        }
    }

    Alibava *B = mngr->get_board();
    if (!beetle->set(name, val & 0xff, B))
        return 0;

    if ( name == "chipMask" )
    {
        if (val>0 && val <=3 )
        {
            beetle->mask(val);
        }
        return 0;
    }
    if ( triggerParam.find(name+'.') != triggerParam.npos)
    {
        TriggerConfig *tc = dynamic_cast<TriggerConfig *>(config->get("trigger"));
        if ( name == "thrsIn1" )
        {
            tc->dacA(val);
        }
        else if (name == "thrsIn2" )
        {
            tc->dacB(val);
        }
        else if (name=="thrsPulse-")
        {
            tc->dacC(val);
        }
        else if (name=="thrsPulse+")
        {
            tc->dacD(val);
        }
        else if (name=="trgIn")
        {
            tc->pulse( !val );
        }
        else if ( name =="trgPulse" )
        {
            tc->pulse(val);
        }
        else if ( name == "trgAND" )
        {
            tc->trigger_and(val);
        }
        else if ( name == "trgOR" )
        {
            tc->trigger_and(!val);
        }
        config->config_trigger(*B);
        return 0;
    }
    if ( compParam.find(name+'.') != compParam.npos )
    {
        int changed_arrays = 0;
        ComparatorConfig comp = beetle->get_comparator_settings();
        if (name=="enableComp")
        {
            comp.setEnable(val);
        }
        else if ( name=="trackMode")
        {
            comp.setMode( val ? ComparatorConfig::trackMode : ComparatorConfig::pulseMode );
        }
        else if ( name=="pulseMode")
        {
            comp.setMode( val ? ComparatorConfig::pulseMode : ComparatorConfig::trackMode );
        }
        else if ( name=="compPolarity")
        {
            comp.setPolarity(0, val==0 ? 0 : 1);
            comp.setPolarity(1, val==0 ? 0 : 1);
        }
        else if ( name=="compPolarity1")
        {
            comp.setPolarity(0, val==0 ? 0 : 1);
        }
        else if ( name=="compPolarity2")
        {
            comp.setPolarity(1, val==0 ? 0 : 1);
        }
        else if ( name == "mainTh")
        {
            comp.setMainTh(0, val);
            comp.setMainTh(1, val);
        }
        else if ( name == "mainTh1")
        {
            comp.setMainTh(0, val);
        }
        else if ( name == "mainTh2")
        {
            comp.setMainTh(1, val);
        }
        else if (name== "deltaTh")
        {
            comp.setDeltaTh(0, val);
            comp.setDeltaTh(1, val);

        }
        else if (name== "deltaTh1")
        {
            comp.setDeltaTh(0, val);

        }
        else if (name== "deltaTh2")
        {
            comp.setDeltaTh(1, val);

        }
        else if (name=="trimCh1")
        {
            std::vector<int> v;
            int rc = get_int_array_value(value, v);
            if ( !rc && v.size()==128 )
            {
                changed_arrays = 1;
                for (int ii=0;ii<128;ii++)
                    comp.setTrim(0, ii, v[ii]);
            }
        }
        else if (name=="trimCh2")
        {
            std::vector<int> v;
            int rc = get_int_array_value(value, v);
            if ( !rc && v.size()==128 )
            {
                changed_arrays = 1;
                for (int ii=0;ii<128;ii++)
                    comp.setTrim(1, ii, v[ii]);
            }
        }
        else if (name=="maskCh1")
        {
            BitSet b;
            b.BitStr(value.size(), value.c_str());
            if (b.nBits()==128)
            {
                changed_arrays = 1;
                comp.setMask(0, b);
            }
        }
        else if (name=="maskCh2")
        {
            BitSet b;
            changed_arrays = 1;
            b.BitStr(value.size(), value.c_str());
            if (b.nBits()==128)
            {
                comp.setMask(1, b);
            }
        }
        beetle->set_comparator_settings(comp);
        config->config_beetle_comparator(*B);


        return 0;
    }
    return 0;
}
