/* -*- mode: c++ -*- */
#ifndef __KeyFile_h__
#define __KeyFile_h__

#include <iosfwd>
#include <exception>
#include <map>
#include <vector>
#include <string>

/**
 * KeyFile is a class to handle key files. This files store the data
 * in groups started by
 *
 * [name_of_he_group]
 *
 * then we have a set of
 *
 * key_name = key_value
 *
 * lines, until the next group is started.
 *
 *
 * blank lines are ignored
 * lines starting with # are comments
 * comments are saved
 * values can be a list of values separated by ';'
 *
 */
struct KeyFilePriv;
class KeyFile
{
    public:
        // Exception
        class Exception : public std::exception
        {
            protected:
                /// stores the exceptin message
                std::string _why;
            public:
                /// Constructor
                Exception();
                Exception(const std::string &s) :
                    _why(s)
                {
                }

                /// Copy constructor
                Exception(const Exception &e);

                /// Destructor
                virtual ~Exception() throw();
                /** Returns a C-style character string describing the general
                 cause of the current error.
                 */
                const char* what() const throw();
        };

    private:
        KeyFilePriv *priv;

    public:
        KeyFile();
        virtual ~KeyFile();

        void reset();

        // check if a current group exists
        bool has_group(const std::string&s) const;

        // check if a given group has a given key
        bool has_key(const std::string &g, const std::string &k) const;

        int add_group(const std::string &g);

        void add_key(const std::string &g, const std::string &k, const std::string &val);

        void del_key(const std::string &g, const std::string &k);

        void del_group(const std::string &g);
        
        std::string value(const std::string &g, const std::string &k) const throw(Exception);

        int load(std::istream &os);
        int save(std::ostream &os) const;

        bool get_as_bool(const std::string &g, const std::string &k) const throw (Exception);
        void get_as_bool_list(const std::string &g, const std::string &k, std::vector<bool> &v) throw (Exception);
        void set_as_bool(const std::string &g, const std::string &k, bool x);
        void set_as_bool_list(const std::string &g, const std::string &k, const std::vector<bool> &v);

        int get_as_int(const std::string &g, const std::string &k) const throw(Exception);
        void get_as_int_list(const std::string &g, const std::string &k, std::vector<int> &v) throw (Exception);
        void set_as_int(const std::string &g, const std::string &k, int val);
        void set_as_int_list(const std::string &g, const std::string &k, const std::vector<int> &v);

        double get_as_double(const std::string &g, const std::string &k) const throw(Exception);
        void get_as_double_list(const std::string &g, const std::string &k, std::vector<double> &v) throw (Exception);
        void set_as_double(const std::string &g, const std::string &k, double val);
        void set_as_double_list(const std::string &g, const std::string &k, const std::vector<double> &v);
};
std::ostream &operator<<(std::ostream &os, const KeyFile &);
std::istream &operator>>(std::istream &is, KeyFile &);

#endif
